#!/usr/bin/perl

use strict;
use warnings;

use Getopt::Long qw(:config no_ignore_case);

my $n = 1; ## counter
my $id = 0;

my $formulae_dir = "formulae";  ## Directory with formulae
my $form_filter = "ltl\$";  ## regular expression that filters the 'ls $formulae_dir' command
my $statopt = ""; ## options given to the script stat.pl

GetOptions(
           'statopt=s' => \$statopt,  ## sets options to stat.pl for each instance
	   'filter=s' => \$form_filter, ## sets the filter for formulae files
           );


my @tools = 
              (
	      "Spin",
	      "Spot",
              "noJumpSpot",
#              "detSpot",
#              "Spot2",
#              "Spot3",
              "LTL3BA",
              "detLTL3BA",
              "LTL2BA",
              "Modella",
              );
my $tmp_file = "stats.tmp";
my $all = "";


### Process one file with LTL properties.  ############################
# * argument $_[0] contains path to the ltl file                      #
# * extracts the name of the model frtom the name of the ltl file     #
# * reads the formulae one by one and generates appropriate commands. #
#######################################################################
sub process_file {

### Parse the name of the model in form name.n.pm, 
### where n is integer parametr - count of instances
### from the formula file name
  my $model_name; my $model_file;
  $_[0] =~ /^(.*\/)*([^.]*)(\.[0-9]+)?\.ltl$/;
  $model_name = $2;
  $model_file = "$2$3.pm";
#  print "$model_name\n$model_file\n";

### Generate commands for each formula in the processed file
  open(LTLFILE,"<","$formulae_dir\/$_[0]") or die ("Can't open $formulae_dir\/$_[0]");
  while(<LTLFILE>) {
    chomp;
    $id++;
    my $formula = $_;
    foreach my $tool (@tools) {
      my $log_file = "log$n.csv";
      $all .= "$log_file ";
      print TMP "$log_file:; ./stat.pl -f \'$formula\' -m $model_name/generated_files/$model_file -t \'$tool\' -l $log_file --id=\'$id\' $statopt\n";
      $n++;
    }
  }
  close LTLFILE;
}

`rm -f $tmp_file Makefile`;
open(TMP,">>$tmp_file");

open(FORM,"-|","ls $formulae_dir | grep $form_filter");
while(<FORM>) {
chomp;
process_file $_;
}
close FORM;
#process_file "driving_phils.1.ltl";

close TMP; 

open(MAKE,">>Makefile");
print MAKE "ALL = $all\n";
print MAKE "spin_results.csv: \$(ALL)\n\t";
print MAKE "echo \'id,model,formula,translator,aut_states,aut_nondet_states,aut_edges,aut_trans,trim_aut_states,trim_aut_edges,trim_aut_trans,stored,visited,matched,trans,depth,error,time,memory,avg_time,min_time,median,timeout,POR,translation command,pr_states,pr_trans,trim_prod_states,trim_prod_trans,pr_scc,equivalent memory usage for states,actual memory usage for states,memory used for hash table,memory used for DFS stack,total actual memory usage' > spin_results.csv;\n\tcat \$(ALL) >>\$\@\n";
print MAKE `cat $tmp_file`;
`rm $tmp_file`;

### Generate clean taret
print MAKE "clean:\n\trm -f \$(ALL) tmp\* -r\n";
close MAKE;
exit 0;
