data <- read.csv('data.csv')

# Gather all columns whose name starts with product_states or product_trans...
names <- colnames(data)
prst_cols <- grep('product_states', names)
prtr_cols <- grep('product_trans', names)

# ... and create new columns with the means of those
data$product_states_mean <- apply(data[,prst_cols], 1, median)
data$product_trans_mean <- apply(data[,prtr_cols], 1, median)

# Omit incomplete lines.
data <- na.omit(data)

# Now output a subset of our columns in a nice LaTeX table

library(tables)
tmp <- booktabs()

x <- latex(file='ltlcross.tex',
           tabular(Heading() * tool ~
                   (n=1) +
                   (Heading('automaton') * (states +
                                            Heading('ndstates') * nondet_states +
                    			    edges +
                    			    Heading('trans') * transitions) +
                    Heading('product') * (Heading('states') * product_states_mean +
                    			  Heading('trans') * product_trans_mean)) * (Heading() * sum),
                    data=data))





# compute a cross-comparison table, showing how many time tool i is
# worse than tool j on the mean number of product  transitions.
library(reshape)

m <- melt(data, id=c('formula', 'tool'), measure.vars=c('product_trans_mean'))
c <- cast(m, formula ~ tool)

tools <- levels(data$tool)
ntools <- length(tools)
a <- array(dim=c(ntools, ntools), dimnames=list(tools, tools))

for (t1 in tools)
  for (t2 in tools)
     a[t1,t2] <- sum(c[t1] > c[t2], na.rm=TRUE);

x <- latex(file='ltlcross2.tex', a)

